module.exports = function(project, options){

    var _  = require('../utils.js')
        , path    = require('path')
        , fs      = require('fs-extra')
        , Views   = require('../views/http.js')
        ;

    var ENDPOINT = 'types';

    /**
     * Fetch the object from the server and write to a file
     */
    function create(env, typeName){
        var client = env.createClient();
        try{
            var model = fs.readJsonFileSync(asFilePath(typeName));
            model.namespace = env.namespace;
            client.post(ENDPOINT, model);
        } catch (err){
            console.log('No such model %s', typeName)
        }
        console.log(client.buildUri(asUrlPath(typeName)));
    }

    /**
     * Show a remote model
     */
    function get(env, typeName, spec){
        env.createClient().get(
            asUrlPath(typeName), Views(spec).entity
        );
    }

    /**
     * Fetch the object from the server and write to a file
     */
    function download(env, typeName){
        env.createClient().pull(asUrlPath(typeName), asFilePath(typeName));
        console.log(asFilePath(typeName));
    }

    /**
     * List the models
     */
    function list(env){
        env.createClient().get(ENDPOINT, function(err, response, body) {
            if(err){
                console.log("Error", err);
            } else {
                var json = JSON.parse(body);
                console.log(_.pluck(json, "name").sort());
            }
        });
    }

    function listProps(env, name){
        env.createClient().get(asUrlPath(name), function(err, response, body) {
                if(err){
                    console.log("Error", err);
                }
                var json = JSON.parse(body);
                var properties = [];
                _.forEach(json.properties, function(value, key, list){
                    if(!_.startsWith(key, 'rbc_') || !_.startsWith(key, 'sys_')){
                        console.log(key);
                    }
                })
        });
    }

    /**
     * Delete from the server
     */
    function destroy(env, typeName){
        return env.createClient().delete(asUrlPath(typeName), Views().status);
    }

    /**
     * Delete locally
     */
    function asFilePath(name){
        return path.join(process.cwd(), 'models', (name + '.json'));
    }

    function asUrlPath(name){
        return [ENDPOINT, name].join('/');
    }

    return {
        post    : create,
        list    : list,
        get     : get,
        download: download,
        'delete': destroy,
        listProps: listProps
    }
};